# Simple Test Launcher

```javascript
require('zrim-test-bootstrap').SimpleTestLauncher
```

## Introduction

A simple class to launch the test. It is in charge of starting jasmine and istanbul.

## Usage

```javascript
const launcher = new SimpleTestLauncher();

const config = {}; // You may use the builder

launcher.configure(config)
.then(() => launcher.run())
.catch(error => {
  // Something went wrong and test failed to launch
});

// The run will automatically call process.exit()
```

## Configuration

Configuration:
- **project** *Object* : Project configuration
  - **rootDirectoryPath** *String* : Root directory for the project
- reports *Object* : Reports configuration
  - rootDirectoryPath *String* : The root directory when to save the reports
- **coverage** *Object*: Coverage configuration
  - enabled *Boolean*: Enable the coverage or not (Default=Env variable *ZE_DISABLE_CODE_COVERAGE* not defined)
  - istanbul *Object*: istanbul configuration
    - excludes *string[]*: Exclude items (Default=['\*\*/test/\*\*', '\*\*/node_modules/\*\*'])
    - nativeConfiguration *Object*: Native istanbul configuration
    - reportDirectory *string*: The report directory (Default=coverage)
  - summary *Object*: Summary to print
    - enabled *Boolean*: Enable the summary or not
    - template *String*: The lodash template for the summary
- **test** *Object*: The test configuration
  - **type** *String* : The test type
  - specDirPath *String*: The spec directory
  - specFileFilters *Array<String|Function|RegExp>*: Spec file filters
  - jasmine *Object*: Jasmine configuration
    - features *Object*: Jasmine features
      - enableJunitXml *Boolean*: Enable or not the junit xml report
      - enableHtml *Boolean*: Enable or not the html report
    - nativeConfig *Object*: A possible native configuration
- steps *Object*: Steps to execute
  - preLaunch *Object[]*: Steps to execute before any run for coverage or test suite
    - **handler** *Function*: The function to call
  - preTestLaunch *Object[]*: Steps to execute before the test suite
    - **handler** *Function*: The function to call
  - postExecution *Object[]*: Steps to execute after the test suite
    - **handler** *Function*: The function to call
  - cleanUp *Object[]*: Steps to execute after everything. It is called even if previous steps failed
    - **handler** *Function*: The function to call
     

**Bold means required**

## Coverage

### Template

We use the lodash template module to generate the summary.
The following variable are given to the template:
- total *Object*
  - percent *Number* : The percentage done (fixed with 2 decimal)
  - pct *Number*: The percentage,
  - total *Number*: The total,
  - covered *Number*: The covered
  - skipped *Number*: The skipped
- statements *Object*: Statements
  - percent *Number*: The percentage
  - covered *NUmber*: The covered
  - total *Number*: The total value
  - skipped *Number*: The skipped
- branches *Object*: Branches
  - percent *Number*: The percentage
  - covered *NUmber*: The covered
  - total *Number*: The total value
  - skipped *Number*: The skipped
- functions *Object*: Functions
  - percent *Number*: The percentage
  - covered *NUmber*: The covered
  - total *Number*: The total value
  - skipped *Number*: The skipped
- lines *Object*: Lines
  - percent *Number*: The percentage
  - covered *NUmber*: The covered
  - total *Number*: The total value
  - skipped *Number*: The skipped
  
## Steps

You may defined different kind of step:
- Before starting coverage and test suite
- Before starting the test suite
- After the test finished
- After everything. Useful to clean up

The function must return a `Promise` object, even if you need to define a 
synchronous method.

The function will be called with a context object unique to the call:
- launcher *SimpleTestLauncher*: The launcher instance
- logger *Object* : A logger to use (debug,info,warn,error)
- executionContext *Object*: An execution context. This is a shared object to all calls.
It will also be pass to jasmine and be available to all your test. It is useful when you
need to pass pre-created value to your test, like database, utilities, etc.

Depending on the step type, the context may contains other properties.

You may return an object as a result to ask the step engine to stop the execution after the step
by returning the property `stopWorkflow=true`.

### Post execution

The post execution context will contains the additional properties:
- testSucceed *Boolean*: Tells you if the test finished with success or not

## Methods

### configure

```
configure(config: Object): Promise<void>
```

Configure the launcher

### run

```
run(): Promise<void>
```

Start the tests. This will call :
- The steps
- The coverage if enabled
- The tests

It will call process.exit at the end of the process with 0 if all succeed, otherwise something
different than 0.

## Environment Variable

The launcher allow you to define environment variable to change the configuration.

- ZE_DISABLE_CODE_COVERAGE : If defined, it will disable the coverage.

## Debug

We use [debug](https://www.npmjs.com/package/debug) with the prefix `ze:test:launcher`.
