
import {SimpleTestLauncherConfigBuilder} from './test-launcher-config-builder';

export class SimpleCoverageConfigBuilder {

  constructor(parent?: SimpleTestLauncherConfigBuilder);

  /**
   * The configuration group name
   */
  public _subConfigurationGroupName: string;

  /**
   * Returns the default configuration
   */
  protected _defaultConfiguration(): Object;

  /**
   * Clear the configuration
   */
  public clear(): SimpleCoverageConfigBuilder;

  /**
   * Enable the coverage
   */
  public enable(): SimpleCoverageConfigBuilder;

  /**
   * Disable the coverage
   */
  public disable(): SimpleCoverageConfigBuilder;

  /**
   * Set the enable value
   * @param [value] The new value. Set undefined to set put the default value
   */
  public setEnabled(value?: boolean|null): SimpleCoverageConfigBuilder;

  /**
   * Set the report directory
   * @param [value] The new value. Set undefined to set put the default value
   */
  public reportDirectory(value?: string|null): SimpleCoverageConfigBuilder;

  /**
   * Set the exclude pattern
   * @param [value] The new value. Set undefined to set put the default value
   */
  public excludePatterns(value?: string[]|string|null): SimpleCoverageConfigBuilder;

  /**
   * Add a exclude patterns
   * @param [value] The patterns to add
   */
  public addExcludePatterns(value?: string[]|string|null): SimpleCoverageConfigBuilder;

  /**
   * Set the native configuration
   * @param [value] The new value. Set undefined to set put the default value
   */
  public nativeConfiguration(value?: Object|null): Object;

  /**
   * Set the summary template.
   *
   * The template is a lodash template.
   * @param [value] The new value. Set undefined to set put the default value
   */
  public summaryTemplate(value?: string): SimpleCoverageConfigBuilder;

  /**
   * Enable or not the summary
   * @param [value] The new value. Set undefined to set put the default value
   */
  public enableSummary(value?: boolean): SimpleCoverageConfigBuilder;

  /**
   * Returns the parent builder
   */
  public parentBuilder(): SimpleTestLauncherConfigBuilder;

  /**
   * Generate the object configuration
   */
  public build(): Object;
}
