const _ = require('lodash');

class SimpleProjectConfigBuilder {

  constructor(parent) {
    // Configure properties object
    Object.defineProperty(this, 'properties', {
      value: {
        parent: parent,
        configuration: this._defaultConfiguration()
      },
      enumerable: false
    });
  }

  get _subConfigurationGroupName() {
    return 'project';
  }

  _defaultConfiguration() {
    return _.cloneDeep({
      rootDirectoryPath: undefined
    });
  }

  clear() {
    this.properties.configuration = this._defaultConfiguration();
    return this;
  }

  rootDirectoryPath(value) {
    if (_.isNil(value)) {
      this.properties.configuration.rootDirectoryPath = this._defaultConfiguration().rootDirectoryPath;
    } else if (_.isString(value)) {
      this.properties.configuration.rootDirectoryPath = value;
    }

    return this;
  }

  parentBuilder() {
    return this.properties.parent;
  }

  build() {
    const userConfig = {
      rootDirectoryPath: this.properties.configuration.rootDirectoryPath
    };

    return _.merge({}, userConfig);
  }
}

exports.SimpleProjectConfigBuilder = module.exports.SimpleProjectConfigBuilder = SimpleProjectConfigBuilder;
