const _ = require('lodash');

class SimpleTestConfigBuilder {

  constructor(parent) {
    // Configure properties object
    Object.defineProperty(this, 'properties', {
      value: {
        parent: parent,
        configuration: this._defaultConfiguration()
      },
      enumerable: false
    });
  }

  get _subConfigurationGroupName() {
    return 'test';
  }

  _defaultConfiguration() {
    return _.cloneDeep({
      type: 'unknown',
      specDirPath: undefined,
      nativeConfiguration: {},
      specFileFilters: [],
      features: {
        enableJunitXml: true,
        enableHtml: true
      }
    });
  }

  clear() {
    this.properties.configuration = this._defaultConfiguration();
    return this;
  }

  specDirPath(value) {
    if (_.isNil(value)) {
      this.properties.configuration.specDirPath = this._defaultConfiguration().specDirPath;
    } else if (_.isString(value)) {
      this.properties.configuration.specDirPath = value;
    }

    return this;
  }

  type(value) {
    if (_.isNil(value)) {
      this.properties.configuration.type = this._defaultConfiguration().type;
    } else if (_.isString(value)) {
      this.properties.configuration.type = value;
    }

    return this;
  }

  unitTest() {
    return this.type('unit');
  }

  integrationTest() {
    return this.type('integration');
  }

  systemTest() {
    return this.type('system');
  }

  nativeConfiguration(value) {
    if (_.isNil(value)) {
      this.properties.configuration.nativeConfiguration = this._defaultConfiguration().nativeConfiguration;
    } else if (_.isObject(value)) {
      this.properties.configuration.nativeConfiguration = value;
    }

    return this;
  }

  withSpecFileFilter(value) {
    if (_.isFunction(value) || _.isRegExp(value) || _.isString(value)) {
      this.properties.configuration.specFileFilters.push(value);
    }

    return this;
  }

  parentBuilder() {
    return this.properties.parent;
  }

  build() {
    const userConfig = {
      type: this.properties.configuration.type,
      specFileFilters: this.properties.configuration.specFileFilters,
      jasmine: {
        features: {
          enableJunitXml: this.properties.configuration.features.enableJunitXml,
          enableHtml: this.properties.configuration.features.enableHtml
        },
        nativeConfig: this.properties.configuration.nativeConfiguration
      }
    };

    return userConfig;
  }
}

exports.SimpleTestConfigBuilder = module.exports.SimpleTestConfigBuilder = SimpleTestConfigBuilder;
