
import {SimpleCoverageConfigBuilder} from './coverage-config-builder';
import {SimpleProjectConfigBuilder} from './project-config-builder';
import {SimpleReportsConfigBuilder} from './reports-config-builder';
import {SimpleTestConfigBuilder} from './test-config-builder';


/**
 * The configuration builder
 * In case you need to add a sub configuration builder, end you property name by ConfigBuilder, and it will
 * automatically added in the clear and build function.
 * To be handled by the build function, the builder need to have the property _subConfigurationGroupName defined
 */
export class SimpleTestLauncherConfigBuilder {

  constructor();

  /**
   * The configuration group name
   */
  public _subConfigurationGroupName: string;

  /**
   * Create the configuration builders
   */
  protected _createSubConfigurationBuilders(): void;

  /**
   * Returns the test configuration
   */
  public testConfiguration(): SimpleTestConfigBuilder;

  /**
   * Returns the reports configuration
   */
  public reportsConfiguration(): SimpleReportsConfigBuilder;

  /**
   * Returns the coverage configuration
   */
  public coverageConfiguration(): SimpleCoverageConfigBuilder;

  /**
   * Returns the project configuration
   */
  public projectConfiguration(): SimpleProjectConfigBuilder;

  /**
   * Clear the configuration
   */
  public clear(): void;

  /**
   * Add a pre launch step function
   * @param value The step function to add
   */
  public withPreLaunchStep(value: Function): SimpleTestLauncherConfigBuilder;

  /**
   * Add a pre test step function
   * @param value The step function to add
   */
  public withPreTestLaunchStep(value: Function): SimpleTestLauncherConfigBuilder;

  /**
   * Add a post execution step function
   * @param value The step function to add
   */
  public withPostExecutionStep(value: Function): SimpleTestLauncherConfigBuilder;

  /**
   * Add a clean up step function
   * @param value The step function to add
   */
  public withCleanUpStep(value: Function): SimpleTestLauncherConfigBuilder;

  /**
   * Generate the configuration
   */
  public build(): Object;
}
