const _ = require('lodash'),
  SimpleCoverageConfigBuilder = require('./coverage-config-builder').SimpleCoverageConfigBuilder,
  SimpleTestConfigBuilder = require('./test-config-builder').SimpleTestConfigBuilder,
  SimpleReportsConfigBuilder = require('./reports-config-builder').SimpleReportsConfigBuilder,
  SimpleProjectConfigBuilder = require('./project-config-builder').SimpleProjectConfigBuilder;


class SimpleTestLauncherConfigBuilder {

  constructor() {
    // Configure properties object
    Object.defineProperty(this, 'properties', {
      value: {
        configuration: this._defaultConfiguration()
      },
      enumerable: false
    });

    this._createSubConfigurationBuilders();
  }

  _createSubConfigurationBuilders() {
    this.properties.coverageConfigBuilder = new SimpleCoverageConfigBuilder(this);
    this.properties.testConfigBuilder = new SimpleTestConfigBuilder(this);
    this.properties.reportsConfigBuilder = new SimpleReportsConfigBuilder(this);
    this.properties.projectConfigBuilder = new SimpleProjectConfigBuilder(this);
  }

  _defaultConfiguration() {
    return {
      steps: {
        preLaunch: [],
        preTestLaunch: [],
        postExecution: [],
        cleanUp: []
      }
    };
  }

  testConfiguration() {
    return this.properties.testConfigBuilder;
  }

  reportsConfiguration() {
    return this.properties.reportsConfigBuilder;
  }

  coverageConfiguration() {
    return this.properties.coverageConfigBuilder;
  }

  projectConfiguration() {
    return this.properties.projectConfigBuilder;
  }

  clear() {
    this.properties.configuration = this._defaultConfiguration();
    // Use a special pattern
    _.each(_.filter(_.keysIn(this.properties), key => _.endsWith(key, 'ConfigBuilder')), key => {
      if (_.isObject(this.properties[key]) && _.isFunction(this.properties[key].clear)) {
        this.properties[key].clear();
      }
    });
    return this;
  }

  withPreLaunchStep(value) {
    if (_.isFunction(value)) {
      this.properties.configuration.steps.preLaunch.push({
        handler: value
      });
    }

    return this;
  }

  withPreTestLaunchStep(value) {
    if (_.isFunction(value)) {
      this.properties.configuration.steps.preTestLaunch.push({
        handler: value
      });
    }

    return this;
  }

  withPostExecutionStep(value) {
    if (_.isFunction(value)) {
      this.properties.configuration.steps.postExecution.push({
        handler: value
      });
    }

    return this;
  }

  withCleanUpStep(value) {
    if (_.isFunction(value)) {
      this.properties.configuration.steps.cleanUp.push({
        handler: value
      });
    }

    return this;
  }

  build() {
    const subConfiguration = {};

    // _subConfigurationGroupName
    _.each(_.filter(_.keysIn(this.properties), key => _.endsWith(key, 'ConfigBuilder')), key => {
      if (_.isObject(this.properties[key]) && _.isFunction(this.properties[key].build) && _.isString(this.properties[key]._subConfigurationGroupName)) {
        subConfiguration[this.properties[key]._subConfigurationGroupName] = this.properties[key].build();
      }
    });

    return _.assign({}, this.properties.configuration, subConfiguration);
  }
}

exports.SimpleTestLauncherConfigBuilder = module.exports.SimpleTestLauncherConfigBuilder = SimpleTestLauncherConfigBuilder;

