const  _ = require('lodash');

/**
 * @typedef {Object} processTemplate~OnResolve
 * @property {string} data The data generated
 */
/**
 * @typedef {Object} processTemplate~Options
 * @property {string} template The template
 * @property {Object} variables The variables
 */
/**
 * Print the coverage summary
 * @param {processTemplate~Options} options The options
 * @return {Promise<processTemplate~OnResolve>}
 */
function processTemplate(options) {
  return new Promise(resolve => {
    // Compile

    const compileOptions = {
      interpolate: /{{([\s\S]+?)}}/g,
      imports: {
        _: _.runInContext()
      }
    };
    const generate = _.template(options.template, compileOptions);

    // Run
    const data = generate(options.variables);
    resolve({
      data: data
    });
  });
}

exports.processTemplate = module.exports.processTemplate = processTemplate;
